"""
    test_catalog
    ~~~~~~~~~~~~

    Test functions that implements catalog related features.

    :copyright: Copyright 2019 by Takayuki SHIMIZUKAWA.
    :license: BSD, see LICENSE for details.
"""
from babel.messages import Catalog, Message


def test_write_and_read_po_file_with_non_ascii_string(temp):
    from sphinx_intl import catalog

    cat = Catalog(locale='ja', domain='domain', fuzzy=False)
    msg = Message('Hello World', 'こんにちは世界')
    cat[msg.id] = msg

    po_file = (temp / 'domain.po')
    catalog.dump_po(po_file, cat)
    cat2 = catalog.load_po(po_file)

    assert cat2[msg.id].string == msg.string


def test_fuzzy_flag_on_catalog_update():
    from sphinx_intl import catalog

    cat = Catalog(locale='ja', domain='domain', fuzzy=False)
    msg = Message('Hello Internationalized Sphinx World !',
                  'こんにちは国際化されたSphinxの世界!')
    cat[msg.id] = msg
    assert not msg.fuzzy

    cat_src = Catalog(locale='en', domain='domain', fuzzy=False)
    msg_src = Message('Hello Internationalized Sphinx World ?')
    cat_src[msg_src.id] = msg_src

    catalog.update_with_fuzzy(cat, cat_src)
    assert msg.id not in cat
    assert cat[msg_src.id].fuzzy



"""
    test_cmd_pot
    ~~~~~~~~~~~~~

    Test pot related commands.

    :copyright: Copyright 2019 by Takayuki SHIMIZUKAWA.
    :license: BSD, see LICENSE for details.
"""

import os

from click.testing import CliRunner

from sphinx_intl import commands

runner = CliRunner()


def test_update_pot_notfound(temp):
    r1 = runner.invoke(commands.update, ['-d', 'locale'])
    assert r1.exit_code != 0
    assert 'Please specify pot directory with -p option,' in r1.output


def test_update_no_language(temp):
    r1 = runner.invoke(commands.update, ['-d', 'locale', '-p', '_build/locale'])
    assert r1.exit_code != 0
    assert 'No languages are found. Please specify language with -l' in r1.output


def test_update_simple(temp):
    r1 = runner.invoke(commands.update, ['-d', 'locale', '-p', '_build/locale', '-l', 'ja'])
    assert r1.exit_code == 0


def test_update_difference_detect(temp):
    r1 = runner.invoke(commands.update, ['-d', 'locale', '-p', '_build/locale', '-l', 'ja'])
    assert r1.exit_code == 0
    assert r1.output.count('Create:') == 1
    assert r1.output.count('Update:') == 0
    assert r1.output.count('Not Changed:') == 0

    with open('_build/locale/README.pot', 'a') as f:
        f.write('\nmsgid "test1"\nmsgstr ""\n')

    r2 = runner.invoke(commands.update, ['-d', 'locale', '-p', '_build/locale'])
    assert r2.exit_code == 0
    assert r2.output.count('Create:') == 0
    assert r2.output.count('Update:') == 1
    assert r2.output.count('Not Changed:') == 0

    with open('_build/locale/README.pot') as f:
        d = f.read()
        d = d.replace('test1', 'test2')
    with open('_build/locale/README.pot', 'w') as f:
        f.write(d)

    r3 = runner.invoke(commands.update, ['-d', 'locale', '-p', '_build/locale'])
    assert r3.exit_code == 0
    assert r3.output.count('Create:') == 0
    assert r3.output.count('Update:') == 1
    assert r3.output.count('Not Changed:') == 0

    r4 = runner.invoke(commands.update, ['-d', 'locale', '-p', '_build/locale'])
    assert r4.exit_code == 0
    assert r4.output.count('Create:') == 0
    assert r4.output.count('Update:') == 0
    assert r4.output.count('Not Changed:') == 1


def test_update_line_width(temp):
    with open('_build/locale/README.pot') as f:
        template = f.read()

    with open('_build/locale/README.pot', 'w') as f:
        f.write(template)
        f.write('\nmsgid "foorbar identifier1"\nmsgstr ""\n')

    po_dir = os.path.join('locale', 'ja', 'LC_MESSAGES')
    po_file = os.path.join(po_dir, 'README.po')

    r1 = runner.invoke(commands.update, ['-d', 'locale', '-p', '_build/locale', '-l', 'ja'])
    assert r1.exit_code == 0

    with open(po_file) as f:
        contents = f.read()
        assert '"foorbar identifier1"\n' in contents

    # change the identifier to trigger an update and impose a lower line-width count
    with open('_build/locale/README.pot', 'w') as f:
        f.write(template)
        f.write('\nmsgid "foorbar identifier2"\nmsgstr ""\n')

    r2 = runner.invoke(commands.update, ['-d', 'locale', '-p', '_build/locale', '-w', '1'])
    assert r2.exit_code == 0

    with open(po_file) as f:
        contents = f.read()
        assert '"foorbar"\n' in contents
        assert '"identifier2"\n' in contents


def test_update_no_obsolete(temp):
    with open('_build/locale/README.pot') as f:
        template = f.read()

    with open('_build/locale/README.pot', 'w') as f:
        f.write(template)
        f.write('\nmsgid "foorbar1"\nmsgstr ""\n')
        f.write('\nmsgid "foorbar2"\nmsgstr ""\n')

    po_dir = os.path.join('locale', 'ja', 'LC_MESSAGES')
    po_file = os.path.join(po_dir, 'README.po')

    r1 = runner.invoke(commands.update, ['-d', 'locale', '-p', '_build/locale', '-l', 'ja'])
    assert r1.exit_code == 0

    with open(po_file) as f:
        contents = f.read()
        assert '\nmsgid "foorbar1"\n' in contents
        assert '\nmsgid "foorbar2"\n' in contents

    # remove the foorbar2 and verify we can see the obsolete entry
    with open('_build/locale/README.pot', 'w') as f:
        f.write(template)
        f.write('\nmsgid "foorbar1"\nmsgstr ""\n')

    r2 = runner.invoke(commands.update, ['-d', 'locale', '-p', '_build/locale'])
    assert r2.exit_code == 0

    with open(po_file) as f:
        contents = f.read()
        assert '\n#~ msgid "foorbar2"\n' in contents

    # remove the foorbar1 and verify we can no longer see any obsolete entry
    with open('_build/locale/README.pot', 'w') as f:
        f.write(template)

    r3 = runner.invoke(commands.update, ['-d', 'locale', '-p', '_build/locale', '--no-obsolete'])
    assert r3.exit_code == 0

    with open(po_file) as f:
        contents = f.read()
        assert 'msgid "foorbar1"' not in contents
        assert 'msgid "foorbar2"' not in contents


def test_stat(temp):
    r1 = runner.invoke(commands.update, ['-d', 'locale', '-p', '_build/locale', '-l', 'ja'])
    assert r1.exit_code == 0

    r2 = runner.invoke(commands.stat, ['-d', 'locale'])
    assert r2.exit_code == 0
    assert 'README.po: 0 translated, 0 fuzzy, 1 untranslated.' in r2.output


def test_stat_with_multiple_languages(temp):
    r1 = runner.invoke(commands.update, ['-d', 'locale', '-p', '_build/locale', '-l', 'ja,de,it'])
    assert r1.exit_code == 0

    # r2 = runner.invoke(commands.stat, ['-d', 'locale', '-l', 'ja,de', '-l', 'it'])
    r2 = runner.invoke(commands.stat, ['-d', 'locale', '-l', 'ja'])
    assert r2.exit_code == 0
    assert 'README.po: 0 translated, 0 fuzzy, 1 untranslated.' in r2.output


def test_build(temp):
    result = runner.invoke(commands.build, ['--locale-dir', 'locale'])
    assert result.exit_code == 0



"""
    test_basic
    ~~~~~~~~~~

    Test functions that implements pot related features.

    :copyright: Copyright 2019 by Takayuki SHIMIZUKAWA.
    :license: BSD, see LICENSE for details.
"""
from unittest import mock

from sphinx_intl import basic


def test_update_simple(temp):
    basic.update('locale', '_build/locale', ('ja',))


def test_update_difference_detect(temp):
    r1 = basic.update('locale', '_build/locale', ('ja',))
    assert r1 == {'create': 1, 'update': 0, 'notchanged': 0}

    with open('_build/locale/README.pot', 'a') as f:
        f.write('\nmsgid "test1"\nmsgstr ""\n')
    r2 = basic.update('locale', '_build/locale', ('ja',))
    assert r2 == {'create': 0, 'update': 1, 'notchanged': 0}

    with open('_build/locale/README.pot') as f:
        d = f.read()
        d = d.replace('test1', 'test2')
    with open('_build/locale/README.pot', 'w') as f:
        f.write(d)
    r3 = basic.update('locale', '_build/locale', ('ja',))
    assert r3 == {'create': 0, 'update': 1, 'notchanged': 0}

    r4 = basic.update('locale', '_build/locale', ('ja',))
    assert r4 == {'create': 0, 'update': 0, 'notchanged': 1}


def test_stat(temp):
    r1 = basic.update('locale', '_build/locale', ('ja',))
    r2 = basic.stat('locale', ('ja',))
    assert r2 == {'locale/ja/LC_MESSAGES/README.po': {'translated': 0, 'fuzzy': 0, 'untranslated': 1}}


def test_stat_with_multiple_languages(temp):
    r1 = basic.update('locale', '_build/locale', ('ja','de','it'))
    r2 = basic.stat('locale', ('ja','de','it'))
    assert r2 == {
        'locale/ja/LC_MESSAGES/README.po': {'translated': 0, 'fuzzy': 0, 'untranslated': 1},
        'locale/de/LC_MESSAGES/README.po': {'translated': 0, 'fuzzy': 0, 'untranslated': 1},
        'locale/it/LC_MESSAGES/README.po': {'translated': 0, 'fuzzy': 0, 'untranslated': 1},
    }


@mock.patch('sphinx_intl.catalog.write_mo')
@mock.patch('sphinx_intl.catalog.load_po')
def test_build(load_po, write_mo, temp):
    basic.update('locale', '_build/locale', ('ja',))
    basic.build('locale', 'locale', ('ja',))
    assert load_po.call_args[0][0].startswith('locale')
    assert load_po.call_args[0][0].endswith('README.po')
    assert write_mo.call_args[0][0].startswith('locale')
    assert write_mo.call_args[0][0].endswith('README.mo')


@mock.patch('sphinx_intl.catalog.write_mo')
@mock.patch('sphinx_intl.catalog.load_po')
def test_build_mo_on_another_location(load_po, write_mo, temp):
    basic.update('locale', '_build/locale', ('ja',))
    basic.build('locale', 'mo_dir', ('ja',))
    assert load_po.call_args[0][0].startswith('locale')
    assert load_po.call_args[0][0].endswith('README.po')
    assert write_mo.call_args[0][0].startswith('mo_dir')
    assert write_mo.call_args[0][0].endswith('README.mo')



"""
    test_transifex
    ~~~~~~~~~~~~~~

    Test functions that implements transifex related features.

    :copyright: Copyright 2019 by Takayuki SHIMIZUKAWA.
    :license: BSD, see LICENSE for details.
"""
import re
from textwrap import dedent

import pytest

from sphinx_intl import transifex


def test_create_transifexrc(home_in_temp):
    transifex.create_transifexrc('spam-token')


def test_create_txconfig(home_in_temp, temp):
    transifex.create_txconfig()


def test_update_txconfig_resources(home_in_temp, temp):
    transifex.create_txconfig()
    transifex.update_txconfig_resources('eggs-org', 'ham-project', 'locale', '_build/locale')


def test_update_txconfig_resources_with_config(home_in_temp, temp):
    tx_dir = temp / '.tx'
    tx_dir.makedirs()
    (tx_dir / 'config').write_text(dedent("""\
    [main]
    host = https://www.transifex.com

    """))

    transifex.update_txconfig_resources('eggs-org', 'ham-project', 'locale', '_build/locale')

    data = (tx_dir / 'config').text().replace('\\', '/')
    assert re.search(r'\[o:eggs-org:p:ham-project:r:README\]', data)
    assert re.search(r'source_file\W*=\W*_build/locale/README.pot', data)


def test_update_txconfig_resources_with_another_pot_dir(home_in_temp, temp):
    tx_dir = temp / '.tx'
    tx_dir.makedirs()
    (tx_dir / 'config').write_text(dedent("""\
    [main]
    host = https://www.transifex.com

    [ham-project.domain1]
    """))

    (temp / '_build' / 'locale').copytree(temp / 'locale' / 'pot')

    transifex.update_txconfig_resources('eggs-org', 'ham-project', 'locale', 'locale/pot')

    data = (tx_dir / 'config').text()
    assert re.search(r'\[o:eggs-org:p:ham-project:r:README\]', data)



def test_update_txconfig_resources_with_project_name_including_dots(home_in_temp, temp):
    tx_dir = temp / '.tx'
    tx_dir.makedirs()
    (tx_dir / 'config').write_text(dedent("""\
    [main]
    host = https://www.transifex.com
    """))

    (temp / '_build' / 'locale').copytree(temp / 'locale' / 'pot')

    transifex.update_txconfig_resources('eggs-org', 'ham-project.com', 'locale', '_build/locale')

    data = (tx_dir / 'config').text()
    assert re.search(r'\[o:eggs-org:p:ham-projectcom:r:README\]', data)


def test_update_txconfig_resources_with_project_name_including_spaces(home_in_temp, temp):
    tx_dir = temp / '.tx'
    tx_dir.makedirs()
    (tx_dir / 'config').write_text(dedent("""\
    [main]
    host = https://www.transifex.com
    """))

    (temp / '_build' / 'locale').copytree(temp / 'locale' / 'pot')

    transifex.update_txconfig_resources('eggs-org', 'ham project com', 'locale', '_build/locale')

    data = (tx_dir / 'config').text()
    assert re.search(r'\[o:eggs-org:p:ham-project-com:r:README\]', data)


def test_update_txconfig_resources_with_potfile_including_symbols(home_in_temp, temp):
    tx_dir = temp / '.tx'
    tx_dir.makedirs()
    (tx_dir / 'config').write_text(dedent("""\
    [main]
    host = https://www.transifex.com
    """))

    (temp / '_build' / 'locale').copytree(temp / 'locale' / 'pot')

    # copy README.pot to 'example document.pot'
    readme = (temp / '_build' / 'locale' / 'README.pot').text()
    (temp / '_build' / 'locale' / 'example document.pot').write_text(readme)

    # copy README.pot to 'test.document.pot'
    (temp / '_build' / 'locale' / 'test.document.pot').write_text(readme)

    transifex.update_txconfig_resources('eggs-org', 'ham project com', 'locale', '_build/locale')

    data = (tx_dir / 'config').text()
    assert re.search(r'\[o:eggs-org:p:ham-project-com:r:example_document\]', data)
    assert re.search(r'\[o:eggs-org:p:ham-project-com:r:test_document\]', data)


@pytest.mark.parametrize("input,expected", [
    ('spam/ham', 'spam--ham'),
    ('spam\\ham', 'spam--ham'),
    ('ham egg.pot', 'ham_egg_pot'),
    ('spam-ham/egg.pot', 'spam-ham--egg_pot'),
    ('glossary', 'glossary_'),
    ('glossary_', 'glossary_'),
    ('settings', 'settings_'),
])
def test_normalize_resource_name(input, expected):
    _callSUT = transifex.normalize_resource_name
    assert _callSUT(input) == expected



"""
    test_cmd_transifex
    ~~~~~~~~~~~~~~~~~~

    Test transifex related commands.

    :copyright: Copyright 2019 by Takayuki SHIMIZUKAWA.
    :license: BSD, see LICENSE for details.
"""
import re
from textwrap import dedent

from click.testing import CliRunner

from sphinx_intl import commands

runner = CliRunner()


def test_create_transifexrc(home_in_temp):
    r1 = runner.invoke(commands.main,
                       [
                           'create-transifexrc',
                           '--transifex-token', 'green-token',
                       ])
    assert r1.exit_code == 0


def test_create_txconfig(home_in_temp, temp):
    r1 = runner.invoke(commands.main, ['create-txconfig'])
    assert r1.exit_code == 0


def test_update_txconfig_resources(home_in_temp, temp):
    r1 = runner.invoke(commands.main, ['create-txconfig'])
    assert r1.exit_code == 0

    r2 = runner.invoke(commands.main,
                       [
                           'create-transifexrc',
                           '--transifex-token', 'green-token',
                       ])
    assert r2.exit_code == 0

    r3 = runner.invoke(commands.update_txconfig_resources,
                       [
                           '--transifex-organization-name', 'eggs-org',
                           '--transifex-project-name', 'ham-project',
                           '-d', 'locale',
                       ])
    assert r3.exit_code == 0


def test_update_txconfig_resources_with_config(home_in_temp, temp):
    tx_dir = temp / '.tx'
    tx_dir.makedirs()
    (tx_dir / 'config').write_text(dedent("""\
    [main]
    host = https://www.transifex.com

    """))

    (temp / '_build' / 'locale').copytree(temp / 'locale' / 'pot')

    r1 = runner.invoke(commands.update_txconfig_resources,
                       [
                           '--transifex-organization-name', 'eggs-org',
                           '--transifex-project-name', 'ham-project',
                           '-d', 'locale',
                       ])
    assert r1.exit_code == 0

    data = (tx_dir / 'config').text()
    assert re.search(r'\[o:eggs-org:p:ham-project:r:README\]', data)
    assert re.search(r'\nresource_name += README\n', data)


def test_update_txconfig_resources_with_pot_dir_argument(home_in_temp, temp):
    tx_dir = temp / '.tx'
    tx_dir.makedirs()
    (tx_dir / 'config').write_text(dedent("""\
    [main]
    host = https://www.transifex.com

    """))

    r1 = runner.invoke(commands.main,
                       ['update-txconfig-resources',
                        '--transifex-organization-name', 'eggs-org',
                        '--transifex-project-name', 'ham-project',
                        '-d', 'locale',
                        '-p', '_build/locale',
                        ])
    assert r1.exit_code == 0

    data = (tx_dir / 'config').text().replace('\\', '/')
    assert re.search(r'\[o:eggs-org:p:ham-project:r:README\]', data)
    assert re.search(r'\nresource_name += README\n', data)
    assert re.search(r'source_file\W*=\W*_build/locale/README.pot', data)


def test_update_txconfig_resources_with_project_name_including_dots(home_in_temp, temp):
    tx_dir = temp / '.tx'
    tx_dir.makedirs()
    (tx_dir / 'config').write_text(dedent("""\
    [main]
    host = https://www.transifex.com
    """))

    (temp / '_build' / 'locale').copytree(temp / 'locale' / 'pot')

    r1 = runner.invoke(commands.main,
                       ['update-txconfig-resources',
                        '--transifex-organization-name', 'eggs-org',
                        '--transifex-project-name', 'ham-project.com',
                        '-d', 'locale',
                        ])
    assert r1.exit_code == 0

    data = (tx_dir / 'config').text()
    assert re.search(r'\[o:eggs-org:p:ham-projectcom:r:README\]', data)
    assert re.search(r'\nresource_name += README\n', data)


def test_update_txconfig_resources_with_project_name_including_spaces(home_in_temp, temp):
    tx_dir = temp / '.tx'
    tx_dir.makedirs()
    (tx_dir / 'config').write_text(dedent("""\
    [main]
    host = https://www.transifex.com
    """))

    (temp / '_build' / 'locale').copytree(temp / 'locale' / 'pot')

    r1 = runner.invoke(commands.main,
                       ['update-txconfig-resources',
                        '-d', 'locale',
                        '--transifex-organization-name', 'eggs-org',
                        '--transifex-project-name', 'ham project com',
                        ])
    assert r1.exit_code == 0

    data = (tx_dir / 'config').text()
    assert re.search(r'\[o:eggs-org:p:ham-project-com:r:README\]', data)
    assert re.search(r'\nresource_name += README\n', data)


def test_update_txconfig_resources_with_potfile_including_symbols(home_in_temp, temp):
    tx_dir = temp / '.tx'
    tx_dir.makedirs()
    (tx_dir / 'config').write_text(dedent("""\
    [main]
    host = https://www.transifex.com
    """))

    (temp / '_build' / 'locale').copytree(temp / 'locale' / 'pot')

    # copy README.pot to 'example document.pot'
    readme = (temp / '_build' / 'locale' / 'README.pot').text()
    (temp / '_build' / 'locale' / 'example document.pot').write_text(readme)

    # copy README.pot to 'test.document.pot'
    (temp / '_build' / 'locale' / 'test.document.pot').write_text(readme)

    r1 = runner.invoke(commands.main,
                       ['update-txconfig-resources',
                        '-d', 'locale',
                        '--transifex-organization-name', 'eggs-org',
                        '--transifex-project-name', 'ham project com',
                        ])
    assert r1.exit_code == 0

    data = (tx_dir / 'config').text()
    assert re.search(r'\[o:eggs-org:p:ham-project-com:r:example_document\]', data)
    assert re.search(r'\[o:eggs-org:p:ham-project-com:r:test_document\]', data)
    assert re.search(r'\nresource_name += example_document\n', data)
    assert re.search(r'\nresource_name += test_document\n', data)


def test_update_txconfig_resources_with_potfile_including_path_separators(home_in_temp, temp):
    tx_dir = temp / '.tx'
    tx_dir.makedirs()
    (tx_dir / 'config').write_text(dedent("""\
    [main]
    host = https://www.transifex.com
    """))

    (temp / '_build' / 'locale').copytree(temp / 'locale' / 'pot')

    # copy README.pot to 'example document.pot'
    readme = (temp / '_build' / 'locale' / 'README.pot').text()
    (temp / '_build' / 'locale' / 'example').makedirs()
    (temp / '_build' / 'locale' / 'example' / 'document.pot').write_text(readme)

    r1 = runner.invoke(commands.main,
                       ['update-txconfig-resources',
                        '-d', 'locale',
                        '--transifex-organization-name', 'eggs-org',
                        '--transifex-project-name', 'ham project com',
                        ])
    assert r1.exit_code == 0

    data = (tx_dir / 'config').text()
    assert re.search(r'\[o:eggs-org:p:ham-project-com:r:example--document\]', data)
    assert re.search(r'\nresource_name += example--document\n', data)
